@@ -1,9 +1,10 @@
 use std::path::PathBuf;
 
 use tokio::task::JoinError;
+use url::Url;
 use zip::result::ZipError;
 
-use crate::metadata::MetadataLoweringError;
+use crate::metadata::MetadataError;
 use distribution_filename::WheelFilenameError;
 use pep440_rs::Version;
 use pypi_types::HashDigest;
@@ -25,6 +26,8 @@ pub enum Error {
     RelativePath(PathBuf),
     #[error(transparent)]
     JoinRelativeUrl(#[from] pypi_types::JoinRelativeError),
+    #[error("Expected a file URL, but received: {0}")]
+    NonFileUrl(Url),
     #[error(transparent)]
     Git(#[from] uv_git::GitResolverError),
     #[error(transparent)]
@@ -70,16 +73,16 @@ pub enum Error {
     NotFound(PathBuf),
     #[error("The source distribution is missing a `PKG-INFO` file")]
     MissingPkgInfo,
-    #[error("The source distribution does not support static metadata in `PKG-INFO`")]
-    DynamicPkgInfo(#[source] pypi_types::MetadataError),
+    #[error("Failed to extract static metadata from `PKG-INFO`")]
+    PkgInfo(#[source] pypi_types::MetadataError),
     #[error("The source distribution is missing a `pyproject.toml` file")]
     MissingPyprojectToml,
-    #[error("The source distribution does not support static metadata in `pyproject.toml`")]
-    DynamicPyprojectToml(#[source] pypi_types::MetadataError),
+    #[error("Failed to extract static metadata from `pyproject.toml`")]
+    PyprojectToml(#[source] pypi_types::MetadataError),
     #[error("Unsupported scheme in URL: {0}")]
     UnsupportedScheme(String),
     #[error(transparent)]
-    MetadataLowering(#[from] MetadataLoweringError),
+    MetadataLowering(#[from] MetadataError),
 
     /// A generic request middleware error happened while making a request.
     /// Refer to the error message for more details.
