@@ -39,7 +39,7 @@ use crate::metadata::{ArchiveMetadata, Metadata};
 use crate::reporter::Facade;
 use crate::source::built_wheel_metadata::BuiltWheelMetadata;
 use crate::source::revision::Revision;
-use crate::Reporter;
+use crate::{Reporter, RequiresDist};
 
 mod built_wheel_metadata;
 mod revision;
@@ -124,6 +124,26 @@ impl<'a, T: BuildContext> SourceDistributionBuilder<'a, T> {
                     }
                 };
 
+                // If the URL is a file URL, use the local path directly.
+                if url.scheme() == "file" {
+                    let path = url
+                        .to_file_path()
+                        .map_err(|()| Error::NonFileUrl(url.clone()))?;
+                    return self
+                        .archive(
+                            source,
+                            &PathSourceUrl {
+                                url: &url,
+                                path: Cow::Owned(path),
+                            },
+                            &cache_shard,
+                            tags,
+                            hashes,
+                        )
+                        .boxed_local()
+                        .await;
+                }
+
                 self.url(
                     source,
                     &dist.file.filename,
@@ -281,6 +301,25 @@ impl<'a, T: BuildContext> SourceDistributionBuilder<'a, T> {
                     }
                 };
 
+                // If the URL is a file URL, use the local path directly.
+                if url.scheme() == "file" {
+                    let path = url
+                        .to_file_path()
+                        .map_err(|()| Error::NonFileUrl(url.clone()))?;
+                    return self
+                        .archive_metadata(
+                            source,
+                            &PathSourceUrl {
+                                url: &url,
+                                path: Cow::Owned(path),
+                            },
+                            &cache_shard,
+                            hashes,
+                        )
+                        .boxed_local()
+                        .await;
+                }
+
                 self.url_metadata(
                     source,
                     &dist.file.filename,
@@ -385,6 +424,14 @@ impl<'a, T: BuildContext> SourceDistributionBuilder<'a, T> {
         Ok(metadata)
     }
 
+    /// Return the [`RequiresDist`] from a `pyproject.toml`, if it can be statically extracted.
+    pub(crate) async fn requires_dist(&self, project_root: &Path) -> Result<RequiresDist, Error> {
+        let requires_dist = read_requires_dist(project_root).await?;
+        let requires_dist =
+            RequiresDist::from_workspace(requires_dist, project_root, self.preview_mode).await?;
+        Ok(requires_dist)
+    }
+
     /// Build a source distribution from a remote URL.
     #[allow(clippy::too_many_arguments)]
     async fn url<'data>(
@@ -1403,7 +1450,16 @@ impl<'a, T: BuildContext> SourceDistributionBuilder<'a, T> {
 
                 return Ok(Some(metadata));
             }
-            Err(err @ (Error::MissingPkgInfo | Error::DynamicPkgInfo(_))) => {
+            Err(
+                err @ (Error::MissingPkgInfo
+                | Error::PkgInfo(
+                    pypi_types::MetadataError::Pep508Error(_)
+                    | pypi_types::MetadataError::DynamicField(_)
+                    | pypi_types::MetadataError::FieldNotFound(_)
+                    | pypi_types::MetadataError::UnsupportedMetadataVersion(_)
+                    | pypi_types::MetadataError::PoetrySyntax,
+                )),
+            ) => {
                 debug!("No static `PKG-INFO` available for: {source} ({err:?})");
             }
             Err(err) => return Err(err),
@@ -1419,7 +1475,16 @@ impl<'a, T: BuildContext> SourceDistributionBuilder<'a, T> {
 
                 return Ok(Some(metadata));
             }
-            Err(err @ (Error::MissingPyprojectToml | Error::DynamicPyprojectToml(_))) => {
+            Err(
+                err @ (Error::MissingPyprojectToml
+                | Error::PyprojectToml(
+                    pypi_types::MetadataError::Pep508Error(_)
+                    | pypi_types::MetadataError::DynamicField(_)
+                    | pypi_types::MetadataError::FieldNotFound(_)
+                    | pypi_types::MetadataError::UnsupportedMetadataVersion(_)
+                    | pypi_types::MetadataError::PoetrySyntax,
+                )),
+            ) => {
                 debug!("No static `pyproject.toml` available for: {source} ({err:?})");
             }
             Err(err) => return Err(err),
@@ -1594,7 +1659,7 @@ async fn read_pkg_info(
     };
 
     // Parse the metadata.
-    let metadata = Metadata23::parse_pkg_info(&content).map_err(Error::DynamicPkgInfo)?;
+    let metadata = Metadata23::parse_pkg_info(&content).map_err(Error::PkgInfo)?;
 
     Ok(metadata)
 }
@@ -1619,12 +1684,30 @@ async fn read_pyproject_toml(
     };
 
     // Parse the metadata.
-    let metadata =
-        Metadata23::parse_pyproject_toml(&content).map_err(Error::DynamicPyprojectToml)?;
+    let metadata = Metadata23::parse_pyproject_toml(&content).map_err(Error::PyprojectToml)?;
 
     Ok(metadata)
 }
 
+/// Return the [`pypi_types::RequiresDist`] from a `pyproject.toml`, if it can be statically extracted.
+async fn read_requires_dist(project_root: &Path) -> Result<pypi_types::RequiresDist, Error> {
+    // Read the `pyproject.toml` file.
+    let pyproject_toml = project_root.join("pyproject.toml");
+    let content = match fs::read_to_string(pyproject_toml).await {
+        Ok(content) => content,
+        Err(err) if err.kind() == std::io::ErrorKind::NotFound => {
+            return Err(Error::MissingPyprojectToml);
+        }
+        Err(err) => return Err(Error::CacheRead(err)),
+    };
+
+    // Parse the metadata.
+    let requires_dist =
+        pypi_types::RequiresDist::parse_pyproject_toml(&content).map_err(Error::PyprojectToml)?;
+
+    Ok(requires_dist)
+}
+
 /// Read an existing cached [`Metadata23`], if it exists.
 async fn read_cached_metadata(cache_entry: &CacheEntry) -> Result<Option<Metadata23>, Error> {
     match fs::read(&cache_entry.path()).await {
