@@ -1,11 +1,15 @@
-use anyhow::Result;
-use cache_key::{CanonicalUrl, RepositoryUrl};
 use std::fmt::Debug;
-use tracing::log::debug;
-use tracing::trace;
+use std::path::Path;
 
-use distribution_types::{InstalledDirectUrlDist, InstalledDist, RequirementSource};
-use pypi_types::{DirInfo, DirectUrl, VcsInfo, VcsKind};
+use anyhow::Result;
+use same_file::is_same_file;
+use serde::Deserialize;
+use tracing::{debug, trace};
+use url::Url;
+
+use cache_key::{CanonicalUrl, RepositoryUrl};
+use distribution_types::{InstalledDirectUrlDist, InstalledDist};
+use pypi_types::{DirInfo, DirectUrl, RequirementSource, VcsInfo, VcsKind};
 use uv_cache::{ArchiveTarget, ArchiveTimestamp};
 
 #[derive(Debug, Copy, Clone)]
@@ -13,6 +17,7 @@ pub(crate) enum RequirementSatisfaction {
     Mismatch,
     Satisfied,
     OutOfDate,
+    Dynamic,
 }
 
 impl RequirementSatisfaction {
@@ -144,9 +149,10 @@ impl RequirementSatisfaction {
                 Ok(Self::Satisfied)
             }
             RequirementSource::Path {
-                path,
-                url: requested_url,
+                install_path: requested_path,
+                lock_path: _,
                 editable: requested_editable,
+                url: _,
             } => {
                 let InstalledDist::Url(InstalledDirectUrlDist { direct_url, .. }) = &distribution
                 else {
@@ -172,25 +178,85 @@ impl RequirementSatisfaction {
                     return Ok(Self::Mismatch);
                 }
 
-                if !CanonicalUrl::parse(installed_url)
-                    .is_ok_and(|installed_url| installed_url == CanonicalUrl::new(requested_url))
+                let Some(installed_path) = Url::parse(installed_url)
+                    .ok()
+                    .and_then(|url| url.to_file_path().ok())
+                else {
+                    return Ok(Self::Mismatch);
+                };
+
+                if !(*requested_path == installed_path
+                    || is_same_file(requested_path, &installed_path).unwrap_or(false))
                 {
                     trace!(
-                        "URL mismatch: {:?} vs. {:?}",
-                        CanonicalUrl::parse(installed_url),
-                        CanonicalUrl::new(requested_url)
+                        "Path mismatch: {:?} vs. {:?}",
+                        requested_path,
+                        installed_path
                     );
-                    return Ok(Self::Mismatch);
+                    return Ok(Self::Satisfied);
                 }
 
-                if !ArchiveTimestamp::up_to_date_with(path, ArchiveTarget::Install(distribution))? {
+                if !ArchiveTimestamp::up_to_date_with(
+                    requested_path,
+                    ArchiveTarget::Install(distribution),
+                )? {
                     trace!("Out of date");
                     return Ok(Self::OutOfDate);
                 }
 
+                // Does the package have dynamic metadata?
+                if is_dynamic(requested_path) {
+                    return Ok(Self::Dynamic);
+                }
+
                 // Otherwise, assume the requirement is up-to-date.
                 Ok(Self::Satisfied)
             }
         }
     }
 }
+
+/// Returns `true` if the source tree at the given path contains dynamic metadata.
+fn is_dynamic(path: &Path) -> bool {
+    // If there's no `pyproject.toml`, we assume it's dynamic.
+    let Ok(contents) = fs_err::read_to_string(path.join("pyproject.toml")) else {
+        return true;
+    };
+    let Ok(pyproject_toml) = toml::from_str::<PyProjectToml>(&contents) else {
+        return true;
+    };
+    // If `[project]` is not present, we assume it's dynamic.
+    let Some(project) = pyproject_toml.project else {
+        // ...unless it appears to be a Poetry project.
+        return pyproject_toml
+            .tool
+            .map_or(true, |tool| tool.poetry.is_none());
+    };
+    // `[project.dynamic]` must be present and non-empty.
+    project.dynamic.is_some_and(|dynamic| !dynamic.is_empty())
+}
+
+/// A pyproject.toml as specified in PEP 517.
+#[derive(Deserialize, Debug)]
+#[serde(rename_all = "kebab-case")]
+struct PyProjectToml {
+    project: Option<Project>,
+    tool: Option<Tool>,
+}
+
+#[derive(Deserialize, Debug)]
+#[serde(rename_all = "kebab-case")]
+struct Project {
+    dynamic: Option<Vec<String>>,
+}
+
+#[derive(Deserialize, Debug)]
+struct Tool {
+    poetry: Option<ToolPoetry>,
+}
+
+#[derive(Deserialize, Debug)]
+struct ToolPoetry {
+    #[allow(dead_code)]
+    name: Option<String>,
+}
