@@ -13,7 +13,7 @@ use tracing::info;
 
 use pypi_types::Scheme;
 use uv_fs::{cachedir, Simplified};
-use uv_interpreter::{Interpreter, VirtualEnvironment};
+use uv_toolchain::{Interpreter, VirtualEnvironment};
 use uv_version::version;
 
 use crate::{Error, Prompt};
@@ -68,12 +68,12 @@ pub fn create_bare_venv(
         // already a "system Python". We canonicalize the path to ensure that it's real and
         // consistent, though we don't expect any symlinks on Windows.
         if interpreter.is_virtualenv() {
-            if let Some(base_executable) = interpreter.base_executable() {
+            if let Some(base_executable) = interpreter.sys_base_executable() {
                 base_executable.to_path_buf()
             } else {
                 // Assume `python.exe`, though the exact executable name is never used (below) on
                 // Windows, only its parent directory.
-                interpreter.base_prefix().join("python.exe")
+                interpreter.sys_base_prefix().join("python.exe")
             }
         } else {
             uv_fs::canonicalize_executable(interpreter.sys_executable())?
@@ -235,12 +235,15 @@ pub fn create_bare_venv(
         ),
         (
             "implementation".to_string(),
-            interpreter.markers().platform_python_implementation.clone(),
+            interpreter
+                .markers()
+                .platform_python_implementation()
+                .to_string(),
         ),
         ("uv".to_string(), version().to_string()),
         (
             "version_info".to_string(),
-            interpreter.markers().python_full_version.string.clone(),
+            interpreter.markers().python_full_version().string.clone(),
         ),
         (
             "include-system-site-packages".to_string(),
@@ -262,9 +265,27 @@ pub fn create_bare_venv(
 
     // Construct the path to the `site-packages` directory.
     let site_packages = location.join(&interpreter.virtualenv().purelib);
+    fs::create_dir_all(&site_packages)?;
+
+    // If necessary, create a symlink from `lib64` to `lib`.
+    // See: https://github.com/python/cpython/blob/b228655c227b2ca298a8ffac44d14ce3d22f6faa/Lib/venv/__init__.py#L135C11-L135C16
+    #[cfg(unix)]
+    if interpreter.pointer_size().is_64()
+        && interpreter.markers().os_name() == "posix"
+        && interpreter.markers().sys_platform() != "darwin"
+    {
+        let lib64 = location.join("lib64");
+        let lib = location.join("lib");
+        match std::os::unix::fs::symlink(lib, lib64) {
+            Ok(()) => {}
+            Err(err) if err.kind() == io::ErrorKind::AlreadyExists => {}
+            Err(err) => {
+                return Err(err.into());
+            }
+        }
+    }
 
     // Populate `site-packages` with a `_virtualenv.py` file.
-    fs::create_dir_all(&site_packages)?;
     fs::write(site_packages.join("_virtualenv.py"), VIRTUALENV_PATCH)?;
     fs::write(site_packages.join("_virtualenv.pth"), "import _virtualenv")?;
 
@@ -374,7 +395,7 @@ fn copy_launcher_windows(
             // `DLLs` subdirectory (if it exists).
             for directory in [
                 python_home,
-                interpreter.base_prefix().join("DLLs").as_path(),
+                interpreter.sys_base_prefix().join("DLLs").as_path(),
             ] {
                 let entries = match fs_err::read_dir(directory) {
                     Ok(read_dir) => read_dir,
