@@ -16,7 +16,9 @@ pub(crate) use pip::list::pip_list;
 pub(crate) use pip::show::pip_show;
 pub(crate) use pip::sync::pip_sync;
 pub(crate) use pip::uninstall::pip_uninstall;
+pub(crate) use project::add::add;
 pub(crate) use project::lock::lock;
+pub(crate) use project::remove::remove;
 pub(crate) use project::run::run;
 pub(crate) use project::sync::sync;
 #[cfg(feature = "self-update")]
