@@ -1,13 +1,13 @@
 use std::fmt::Write;
 
-use anyhow::Result;
+use anyhow::{Context, Result};
 use itertools::Itertools;
 use owo_colors::OwoColorize;
 use tracing::debug;
 
 use distribution_types::{IndexLocations, Resolution};
 use install_wheel_rs::linker::LinkMode;
-use pep440_rs::{Version, VersionSpecifiers};
+use pep440_rs::Version;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, RegistryClientBuilder};
 use uv_configuration::{
@@ -15,29 +15,32 @@ use uv_configuration::{
     SetupPyStrategy, Upgrade,
 };
 use uv_dispatch::BuildDispatch;
-use uv_distribution::ProjectWorkspace;
+use uv_distribution::Workspace;
 use uv_fs::Simplified;
 use uv_git::GitResolver;
 use uv_installer::{SatisfiesResult, SitePackages};
-use uv_interpreter::{find_default_interpreter, PythonEnvironment};
 use uv_requirements::{RequirementsSource, RequirementsSpecification};
-use uv_resolver::{FlatIndex, InMemoryIndex, Options};
+use uv_resolver::{FlatIndex, InMemoryIndex, Options, RequiresPython};
+use uv_toolchain::{PythonEnvironment, SystemPython, Toolchain, ToolchainRequest, VersionRequest};
 use uv_types::{BuildIsolation, HashStrategy, InFlight};
+use uv_warnings::warn_user;
 
 use crate::commands::pip;
 use crate::printer::Printer;
 
+pub(crate) mod add;
 pub(crate) mod lock;
+pub(crate) mod remove;
 pub(crate) mod run;
 pub(crate) mod sync;
 
 #[derive(thiserror::Error, Debug)]
 pub(crate) enum ProjectError {
     #[error("The current Python version ({0}) is not compatible with the locked Python requirement ({1})")]
-    RequiresPython(Version, VersionSpecifiers),
+    PythonIncompatibility(Version, RequiresPython),
 
     #[error(transparent)]
-    Interpreter(#[from] uv_interpreter::Error),
+    Interpreter(#[from] uv_toolchain::Error),
 
     #[error(transparent)]
     Virtualenv(#[from] uv_virtualenv::Error),
@@ -45,6 +48,9 @@ pub(crate) enum ProjectError {
     #[error(transparent)]
     Tags(#[from] platform_tags::TagsError),
 
+    #[error(transparent)]
+    FlatIndex(#[from] uv_client::FlatIndexError),
+
     #[error(transparent)]
     Lock(#[from] uv_resolver::LockError),
 
@@ -62,51 +68,119 @@ pub(crate) enum ProjectError {
 
     #[error(transparent)]
     Operation(#[from] pip::operations::Error),
+
+    #[error(transparent)]
+    RequiresPython(#[from] uv_resolver::RequiresPythonError),
+}
+
+/// Find the virtual environment for the current project.
+pub(crate) fn find_environment(
+    workspace: &Workspace,
+    cache: &Cache,
+) -> Result<PythonEnvironment, uv_toolchain::Error> {
+    PythonEnvironment::from_root(workspace.venv(), cache)
 }
 
 /// Initialize a virtual environment for the current project.
 pub(crate) fn init_environment(
-    project: &ProjectWorkspace,
+    workspace: &Workspace,
+    python: Option<&str>,
     preview: PreviewMode,
     cache: &Cache,
     printer: Printer,
 ) -> Result<PythonEnvironment, ProjectError> {
-    let venv = project.workspace().root().join(".venv");
+    let venv = workspace.root().join(".venv");
+
+    let requires_python = workspace
+        .root_member()
+        .and_then(|root| root.project().requires_python.as_ref());
 
     // Discover or create the virtual environment.
-    // TODO(charlie): If the environment isn't compatible with `--python`, recreate it.
-    match PythonEnvironment::from_root(&venv, cache) {
-        Ok(venv) => Ok(venv),
-        Err(uv_interpreter::Error::NotFound(_)) => {
-            // TODO(charlie): Respect `--python`; if unset, respect `Requires-Python`.
-            let interpreter = find_default_interpreter(preview, cache)
-                .map_err(uv_interpreter::Error::from)?
-                .map_err(uv_interpreter::Error::from)?
-                .into_interpreter();
+    match PythonEnvironment::from_root(venv, cache) {
+        Ok(venv) => {
+            // `--python` has highest precedence, after that we check `requires_python` from
+            // `pyproject.toml`. If `--python` and `requires_python` are mutually incompatible,
+            // we'll fail at the build or at last the install step when we aren't able to install
+            // the editable wheel for the current project into the venv.
+            // TODO(konsti): Do we want to support a workspace python version requirement?
+            let is_satisfied = if let Some(python) = python {
+                ToolchainRequest::parse(python).satisfied(venv.interpreter(), cache)
+            } else if let Some(requires_python) = requires_python {
+                requires_python.contains(venv.interpreter().python_version())
+            } else {
+                true
+            };
+
+            if is_satisfied {
+                return Ok(venv);
+            }
 
             writeln!(
                 printer.stderr(),
-                "Using Python {} interpreter at: {}",
-                interpreter.python_version(),
-                interpreter.sys_executable().user_display().cyan()
+                "Removing virtual environment at: {}",
+                venv.root().user_display().cyan()
             )?;
-
-            writeln!(
-                printer.stderr(),
-                "Creating virtualenv at: {}",
-                venv.user_display().cyan()
-            )?;
-
-            Ok(uv_virtualenv::create_venv(
-                &venv,
-                interpreter,
-                uv_virtualenv::Prompt::None,
-                false,
-                false,
-            )?)
+            fs_err::remove_dir_all(venv.root())
+                .context("Failed to remove existing virtual environment")?;
         }
-        Err(e) => Err(e.into()),
+        Err(uv_toolchain::Error::NotFound(_)) => {}
+        Err(e) => return Err(e.into()),
     }
+
+    let interpreter = if let Some(request) = python.map(ToolchainRequest::parse).or(requires_python
+        .map(|specifiers| ToolchainRequest::Version(VersionRequest::Range(specifiers.clone()))))
+    {
+        Toolchain::find_requested(
+            &request,
+            // Otherwise we'll try to use the venv we just deleted.
+            SystemPython::Required,
+            preview,
+            cache,
+        )
+    } else {
+        Toolchain::find(
+            None,
+            // Otherwise we'll try to use the venv we just deleted.
+            SystemPython::Required,
+            preview,
+            cache,
+        )
+    }?
+    .into_interpreter();
+
+    if let Some(requires_python) = requires_python {
+        if !requires_python.contains(interpreter.python_version()) {
+            warn_user!(
+                "The Python {} you requested with {} is incompatible with the requirement of the \
+                project of {}",
+                interpreter.python_version(),
+                python.unwrap_or("(default)"),
+                requires_python
+            );
+        }
+    }
+
+    writeln!(
+        printer.stderr(),
+        "Using Python {} interpreter at: {}",
+        interpreter.python_version(),
+        interpreter.sys_executable().user_display().cyan()
+    )?;
+
+    let venv = workspace.venv();
+    writeln!(
+        printer.stderr(),
+        "Creating virtualenv at: {}",
+        venv.user_display().cyan()
+    )?;
+
+    Ok(uv_virtualenv::create_venv(
+        &venv,
+        interpreter,
+        uv_virtualenv::Prompt::None,
+        false,
+        false,
+    )?)
 }
 
 /// Update a [`PythonEnvironment`] to satisfy a set of [`RequirementsSource`]s.
@@ -223,7 +297,7 @@ pub(crate) async fn update_environment(
         &reinstall,
         &upgrade,
         interpreter,
-        tags,
+        Some(tags),
         Some(markers),
         None,
         &client,
