@@ -1,4 +1,5 @@
 use anyhow::Result;
+use std::path::Path;
 
 use distribution_types::IndexLocations;
 use install_wheel_rs::linker::LinkMode;
@@ -12,6 +13,7 @@ use uv_dispatch::BuildDispatch;
 use uv_distribution::{ProjectWorkspace, DEV_DEPENDENCIES};
 use uv_git::GitResolver;
 use uv_installer::SitePackages;
+use uv_normalize::PackageName;
 use uv_resolver::{FlatIndex, InMemoryIndex, Lock};
 use uv_toolchain::PythonEnvironment;
 use uv_types::{BuildIsolation, HashStrategy, InFlight};
@@ -58,7 +60,8 @@ pub(crate) async fn sync(
 
     // Perform the sync operation.
     do_sync(
-        &project,
+        project.project_name(),
+        project.workspace().root(),
         &venv,
         &lock,
         &index_locations,
@@ -76,7 +79,8 @@ pub(crate) async fn sync(
 /// Sync a lockfile with an environment.
 #[allow(clippy::too_many_arguments)]
 pub(super) async fn do_sync(
-    project: &ProjectWorkspace,
+    project_name: &PackageName,
+    workspace_root: &Path,
     venv: &PythonEnvironment,
     lock: &Lock,
     index_locations: &IndexLocations,
@@ -107,7 +111,8 @@ pub(super) async fn do_sync(
     let tags = venv.interpreter().tags()?;
 
     // Read the lockfile.
-    let resolution = lock.to_resolution(markers, tags, project.project_name(), &extras, &dev)?;
+    let resolution =
+        lock.to_resolution(workspace_root, markers, tags, project_name, &extras, &dev)?;
 
     // Initialize the registry client.
     // TODO(zanieb): Support client options e.g. offline, tls, etc.
